// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "DawnPokerGame.hpp"

using namespace Dawn;

DawnGame::DawnGame(DawnHost &host) :
  host(host),
  renderManager(*this)
{
}

int32_t DawnGame::init() {
  this->assetManager.init();
  this->renderManager.init();

  this->scene = std::make_shared<Scene>(*this);

  auto cameraObject = this->scene->createSceneItem();
  auto camera = cameraObject->addComponent<Camera>();
  camera->transform.lookAt(glm::vec3(5, 5, 5), glm::vec3(0, 0, 0));
  
  auto canvas = UICanvas::createCanvas(this->scene);
  auto test = canvas->addElement<UISprite>();
  test->setTransform(
    UI_COMPONENT_ALIGN_START,
    UI_COMPONENT_ALIGN_START,
    glm::vec4(0, 0, 32, 32),
    0
  );

  return DAWN_GAME_INIT_RESULT_SUCCESS;
}

int32_t DawnGame::update(float_t delta) {
  this->assetManager.update();

  if(this->scene != nullptr) this->scene->update();
  
  this->renderManager.update();

  return DAWN_GAME_UPDATE_RESULT_SUCCESS;
}

DawnGame::~DawnGame() {
  
}